@@ -18,11 +18,12 @@ from typing import (
     Optional,
     Tuple,
     Type,
+    TypedDict,
     Union,
     cast,
 )
 
-from ....utils.async_itertools import async_chain, async_chain_iterator, async_next
+from ....utils.async_itertools import async_chain, async_chain_iterator
 from ....utils.async_tools import threaded
 from ....utils.logging import LoggingDescriptor
 from ...common.decorators import language_id, trigger_characters
@@ -126,7 +127,6 @@ class RobotCompletionProtocolPart(RobotLanguageServerProtocolPart):
     @threaded()
     @_logger.call
     async def resolve(self, sender: Any, completion_item: CompletionItem) -> CompletionItem:
-
         if completion_item.data is not None:
             document_uri = completion_item.data.get("document_uri", None)
             if document_uri is not None:
@@ -201,6 +201,26 @@ def get_reserved_keywords() -> List[str]:
     return __reserved_keywords
 
 
+class CompletionItemData(TypedDict):
+    document_uri: str
+    type: str
+    name: str
+
+
+class CompletionKeywordData(CompletionItemData):
+    libname: Optional[str]
+    hash: str
+    id: str
+
+
+class CompletionItemImportData(CompletionItemData):
+    import_name: str
+    args: Tuple[Any, ...]
+    alias: Optional[str]
+    hash: str
+    id: str
+
+
 class CompletionCollector(ModelHelperMixin):
     _logger = LoggingDescriptor()
 
@@ -260,22 +280,70 @@ class CompletionCollector(ModelHelperMixin):
         return result
 
     async def resolve(self, completion_item: CompletionItem) -> CompletionItem:
-        if completion_item.data is not None:
-            document_uri = completion_item.data.get("document_uri", None)
+        data = cast(CompletionItemData, completion_item.data)
+
+        if data is not None:
+            document_uri = data.get("document_uri", None)
             if document_uri is not None:
                 document = await self.parent.documents.get(document_uri)
-                if document is not None and (type := completion_item.data.get("type", None)) is not None:
-                    if type in [
+                if document is not None and (comp_type := data.get("type", None)) is not None:
+                    if comp_type in [
                         CompleteResultKind.MODULE.name,
                         CompleteResultKind.MODULE_INTERNAL.name,
                         CompleteResultKind.FILE.name,
                     ]:
-                        import_name = completion_item.data.get("import_name", None)
-                        args = tuple(completion_item.data.get("args", ()))
-                        alias = completion_item.data.get("alias", None)
-                        if import_name is not None:
+                        if (lib_id := data.get("id", None)) is not None:
                             try:
-                                lib_doc = await self.namespace.get_imported_library_libdoc(import_name, args, alias)
+                                lib_doc = next(
+                                    (
+                                        ld.library_doc
+                                        for ld in (await self.namespace.get_libraries()).values()
+                                        if str(id(ld.library_doc)) == lib_id
+                                    ),
+                                    None,
+                                )
+                                if lib_doc is not None:
+                                    completion_item.documentation = MarkupContent(
+                                        kind=MarkupKind.MARKDOWN, value=lib_doc.to_markdown(False)
+                                    )
+
+                            except (SystemExit, KeyboardInterrupt, asyncio.CancelledError):
+                                raise
+                            except BaseException as e:
+                                completion_item.documentation = MarkupContent(
+                                    kind=MarkupKind.MARKDOWN, value=f"Error:\n{e}"
+                                )
+                        elif (name := data.get("name", None)) is not None:
+                            try:
+                                lib_doc = await self.namespace.imports_manager.get_libdoc_for_library_import(
+                                    name,
+                                    (),
+                                    str(document.uri.to_path().parent),
+                                    variables=await self.namespace.get_resolvable_variables(),
+                                )
+
+                                if lib_doc is not None:
+                                    completion_item.documentation = MarkupContent(
+                                        kind=MarkupKind.MARKDOWN, value=lib_doc.to_markdown(False)
+                                    )
+
+                            except (SystemExit, KeyboardInterrupt):
+                                raise
+                            except BaseException:
+                                pass
+
+                    elif comp_type in [CompleteResultKind.RESOURCE.name]:
+
+                        if (res_id := data.get("id", None)) is not None:
+                            try:
+                                lib_doc = next(
+                                    (
+                                        ld.library_doc
+                                        for ld in (await self.namespace.get_resources()).values()
+                                        if str(id(ld.library_doc)) == res_id
+                                    ),
+                                    None,
+                                )
 
                                 if lib_doc is not None:
                                     completion_item.documentation = MarkupContent(
@@ -288,29 +356,30 @@ class CompletionCollector(ModelHelperMixin):
                                 completion_item.documentation = MarkupContent(
                                     kind=MarkupKind.MARKDOWN, value=f"Error:\n{e}"
                                 )
-                    elif type in [CompleteResultKind.RESOURCE.name]:
-                        import_name = completion_item.data.get("import_name", None)
-                        if import_name is not None:
+
+                        elif (name := data.get("name", None)) is not None:
                             try:
-                                lib_doc = await self.namespace.get_imported_resource_libdoc(import_name)
+                                lib_doc = await self.namespace.imports_manager.get_libdoc_for_resource_import(
+                                    name,
+                                    str(document.uri.to_path().parent),
+                                    variables=await self.namespace.get_resolvable_variables(),
+                                )
 
                                 if lib_doc is not None:
                                     completion_item.documentation = MarkupContent(
-                                        kind=MarkupKind.MARKDOWN, value=lib_doc.to_markdown()
+                                        kind=MarkupKind.MARKDOWN, value=lib_doc.to_markdown(False)
                                     )
 
-                            except (SystemExit, KeyboardInterrupt, asyncio.CancelledError):
+                            except (SystemExit, KeyboardInterrupt):
                                 raise
-                            except BaseException as e:
-                                completion_item.documentation = MarkupContent(
-                                    kind=MarkupKind.MARKDOWN, value=f"Error:\n{e}"
-                                )
-                    elif type in [CompleteResultKind.KEYWORD.name]:
-                        id = completion_item.data.get("id", None)
-                        if id is not None:
+                            except BaseException:
+                                pass
+                    elif comp_type in [CompleteResultKind.KEYWORD.name]:
+                        kw_id = data.get("id", None)
+                        if kw_id is not None:
                             try:
-                                kw_doc = await async_next(
-                                    (kw async for kw in self.namespace.iter_all_keywords() if hash(kw) == id),
+                                kw_doc = next(
+                                    (kw for kw in await self.namespace.get_keywords() if str(id(kw)) == kw_id),
                                     None,
                                 )
 
@@ -470,6 +539,24 @@ class CompletionCollector(ModelHelperMixin):
             for setting in settings
         ]
 
+    async def create_bdd_prefix_completion_items(self, range: Optional[Range]) -> List[CompletionItem]:
+        prefixes = {"Given", "When", "Then", "And", "But"}
+
+        if self.namespace.languages is not None:
+            prefixes.update(self.namespace.languages.bdd_prefixes)
+
+        return [
+            CompletionItem(
+                label=prefix,
+                kind=CompletionItemKind.UNIT,
+                detail="BDD Prefix",
+                sort_text=f"080_{prefix}",
+                insert_text_format=InsertTextFormat.PLAINTEXT,
+                text_edit=TextEdit(range=range, new_text=f"{prefix} ") if range is not None else None,
+            )
+            for prefix in prefixes
+        ]
+
     async def create_keyword_settings_completion_items(self, range: Optional[Range]) -> List[CompletionItem]:
         from robot.parsing.lexer.settings import KeywordSettings
 
@@ -525,6 +612,7 @@ class CompletionCollector(ModelHelperMixin):
         add_reserverd: bool = True,
         add_none: bool = False,
         in_template: bool = False,
+        add_bdd_prefixes: bool = True,
     ) -> List[CompletionItem]:
         result: List[CompletionItem] = []
         if self.document is None:
@@ -532,6 +620,22 @@ class CompletionCollector(ModelHelperMixin):
 
         r: Optional[Range] = None
 
+        has_bdd = False
+        bdd_token = None
+
+        if token is not None:
+            old_token = token
+            bdd_token, token = self.split_bdd_prefix(self.namespace, token)
+
+            if token is not None and token.value == "":
+                token = None
+
+            if bdd_token is not None and position.character > range_from_token(bdd_token).end.character:
+                has_bdd = True
+
+            if not has_bdd and token is None:
+                token = old_token
+
         if token is not None:
             r = range_from_token(token)
 
@@ -581,13 +685,14 @@ class CompletionCollector(ModelHelperMixin):
                                     )
                                     if r is not None
                                     else None,
-                                    data={
-                                        "document_uri": str(self.document.uri),
-                                        "type": CompleteResultKind.KEYWORD.name,
-                                        "libname": kw.libname,
-                                        "name": kw.name,
-                                        "id": hash(kw),
-                                    },
+                                    data=CompletionKeywordData(
+                                        document_uri=str(self.document.uri),
+                                        type=CompleteResultKind.KEYWORD.name,
+                                        libname=kw.libname,
+                                        name=kw.name,
+                                        hash=str(hash(kw)),
+                                        id=str(id(kw)),
+                                    ),
                                 )
                             )
 
@@ -620,13 +725,14 @@ class CompletionCollector(ModelHelperMixin):
                                             if not kw.is_embedded
                                             else self.get_keyword_snipped_text(kw, in_template),
                                         ),
-                                        data={
-                                            "document_uri": str(self.document.uri),
-                                            "type": CompleteResultKind.KEYWORD.name,
-                                            "libname": kw.libname,
-                                            "name": kw.name,
-                                            "id": hash(kw),
-                                        },
+                                        data=CompletionKeywordData(
+                                            document_uri=str(self.document.uri),
+                                            type=CompleteResultKind.KEYWORD.name,
+                                            libname=kw.libname,
+                                            name=kw.name,
+                                            hash=str(hash(kw)),
+                                            id=str(id(kw)),
+                                        ),
                                     )
                                 )
 
@@ -651,13 +757,14 @@ class CompletionCollector(ModelHelperMixin):
                         range=r,
                         new_text=kw.name if not kw.is_embedded else self.get_keyword_snipped_text(kw, in_template),
                     ),
-                    data={
-                        "document_uri": str(self.document.uri),
-                        "type": CompleteResultKind.KEYWORD.name,
-                        "libname": kw.libname,
-                        "name": kw.name,
-                        "id": hash(kw),
-                    },
+                    data=CompletionKeywordData(
+                        document_uri=str(self.document.uri),
+                        type=CompleteResultKind.KEYWORD.name,
+                        libname=kw.libname,
+                        name=kw.name,
+                        hash=str(hash(kw)),
+                        id=str(id(kw)),
+                    ),
                 )
             )
 
@@ -671,14 +778,16 @@ class CompletionCollector(ModelHelperMixin):
                     deprecated=v.library_doc.is_deprecated,
                     insert_text_format=InsertTextFormat.PLAINTEXT,
                     text_edit=TextEdit(range=r, new_text=k),
-                    data={
-                        "document_uri": str(self.document.uri),
-                        "type": CompleteResultKind.MODULE.name,
-                        "name": v.name,
-                        "import_name": v.import_name,
-                        "args": v.args,
-                        "alias": v.alias,
-                    },
+                    data=CompletionItemImportData(
+                        document_uri=str(self.document.uri),
+                        type=CompleteResultKind.MODULE.name,
+                        name=v.name,
+                        import_name=v.import_name,
+                        args=v.args,
+                        alias=v.alias,
+                        id=str(id(v.library_doc)),
+                        hash=str(hash(v.library_doc)),
+                    ),
                 )
             )
 
@@ -692,12 +801,16 @@ class CompletionCollector(ModelHelperMixin):
                     sort_text=f"030_{v.name}",
                     insert_text_format=InsertTextFormat.PLAINTEXT,
                     text_edit=TextEdit(range=r, new_text=v.name),
-                    data={
-                        "document_uri": str(self.document.uri),
-                        "type": CompleteResultKind.RESOURCE.name,
-                        "name": k,
-                        "import_name": v.import_name,
-                    },
+                    data=CompletionItemImportData(
+                        document_uri=str(self.document.uri),
+                        type=CompleteResultKind.RESOURCE.name,
+                        name=k,
+                        import_name=v.import_name,
+                        args=(),
+                        alias=None,
+                        id=str(id(v.library_doc)),
+                        hash=str(hash(v.library_doc)),
+                    ),
                 )
             )
 
@@ -712,6 +825,11 @@ class CompletionCollector(ModelHelperMixin):
                 )
             )
 
+        if add_bdd_prefixes and not has_bdd:
+            result += await self.create_bdd_prefix_completion_items(
+                range_from_token(token) if token is not None else None
+            )
+
         if add_reserverd:
             for k in get_reserved_keywords():
                 result.append(
@@ -938,8 +1056,6 @@ class CompletionCollector(ModelHelperMixin):
             if len(statement_node.tokens) > index:
                 token = statement_node.tokens[index]
 
-                token = self.strip_bdd_prefix(self.namespace, token)
-
                 r = range_from_token(token)
                 if position.is_in_range(r):
                     return await create_items(in_assign, in_template, r, token, position)
@@ -956,7 +1072,7 @@ class CompletionCollector(ModelHelperMixin):
                             in_assign,
                             in_template,
                             r,
-                            None if self.is_bdd_token(self.namespace, token) else token,
+                            token,
                             position,
                         )
 
@@ -1084,7 +1200,7 @@ class CompletionCollector(ModelHelperMixin):
             r = range_from_token(token)
             if position.is_in_range(r):
                 return await self.create_keyword_completion_items(
-                    None if self.is_bdd_token(self.namespace, token) else token,
+                    token,
                     position,
                     add_reserverd=False,
                     add_none=True,
@@ -1100,7 +1216,7 @@ class CompletionCollector(ModelHelperMixin):
             r.end.character += 1
             if position.is_in_range(r):
                 return await self.create_keyword_completion_items(
-                    None if self.is_bdd_token(self.namespace, token) else token,
+                    token,
                     position,
                     add_reserverd=False,
                     add_none=True,
@@ -1227,7 +1343,7 @@ class CompletionCollector(ModelHelperMixin):
             r.end.character += 1
             if position.is_in_range(r):
                 return await self.create_keyword_completion_items(
-                    None if self.is_bdd_token(self.namespace, token) else token,
+                    token,
                     position,
                     add_reserverd=False,
                     add_none=True,
@@ -1336,8 +1452,6 @@ class CompletionCollector(ModelHelperMixin):
                 else None
             )
 
-            sep = text_before_position[last_separator_index] if last_separator_index < len(text_before_position) else ""
-
             try:
                 complete_list = await self.namespace.imports_manager.complete_library_import(
                     first_part if first_part else None,
@@ -1371,11 +1485,11 @@ class CompletionCollector(ModelHelperMixin):
                     sort_text=f"030_{e}",
                     insert_text_format=InsertTextFormat.PLAINTEXT,
                     text_edit=TextEdit(range=r, new_text=e.label) if r is not None else None,
-                    data={
-                        "document_uri": str(self.document.uri),
-                        "type": e.kind.name,
-                        "name": ((first_part + sep) if first_part is not None else "") + e.label,
-                    },
+                    data=CompletionItemData(
+                        document_uri=str(self.document.uri),
+                        type=e.kind.name,
+                        name=((first_part) if first_part is not None else "") + e.label,
+                    ),
                 )
                 for e in complete_list
             ]
@@ -1466,11 +1580,11 @@ class CompletionCollector(ModelHelperMixin):
                             filter_text=e.name,
                             insert_text_format=InsertTextFormat.PLAINTEXT,
                             text_edit=TextEdit(range=completion_range, new_text=f"{e.name}="),
-                            data={
-                                "document_uri": str(self.document.uri),
-                                "type": "Argument",
-                                "name": e,
-                            },
+                            data=CompletionItemData(
+                                document_uri=str(self.document.uri),
+                                type="Argument",
+                                name=e.name,
+                            ),
                         )
                         for i, e in enumerate(init.args)
                         if e.kind
@@ -1612,11 +1726,11 @@ class CompletionCollector(ModelHelperMixin):
                 sort_text=f"030_{e}",
                 insert_text_format=InsertTextFormat.PLAINTEXT,
                 text_edit=TextEdit(range=r, new_text=e.label) if r is not None else None,
-                data={
-                    "document_uri": str(self.document.uri),
-                    "type": e.kind.name,
-                    "name": ((first_part) if first_part is not None else "") + e.label,
-                },
+                data=CompletionItemData(
+                    document_uri=str(self.document.uri),
+                    type=e.kind.name,
+                    name=((first_part) if first_part is not None else "") + e.label,
+                ),
             )
             for e in complete_list
         ]
@@ -1718,11 +1832,11 @@ class CompletionCollector(ModelHelperMixin):
                 sort_text=f"030_{e}",
                 insert_text_format=InsertTextFormat.PLAINTEXT,
                 text_edit=TextEdit(range=r, new_text=e.label) if r is not None else None,
-                data={
-                    "document_uri": str(self.document.uri),
-                    "type": e.kind.name,
-                    "name": ((first_part) if first_part is not None else "") + e.label,
-                },
+                data=CompletionItemData(
+                    document_uri=str(self.document.uri),
+                    type=e.kind.name,
+                    name=((first_part) if first_part is not None else "") + e.label,
+                ),
             )
             for e in complete_list
         ]
